#include    "rc_def.h"
#include    "comhead.h"
#pragma hdrstop

//hpp***************************************************************************
#include <time.h>
//pph***************************************************************************

//hpp***************************************************************************
extern FILE *trace;
extern time_t *zidx;
//pph***************************************************************************

// m_topo.cpp
// (C) 1993,1994 by Heiko Damerow MPG AG "Rntgenbeugung"
// Modul Topographie
#include		"m_devcom.h"
#include		"m_layer.h"
#include    "m_steerg.h"
#include    "m_topo.h"

#define 		M_TimeString(buffer,TTicks) 											\
										(sprintf(	(LPSTR)buffer,"%02d:%02d:%02d", \
															(int)(TTicks/3600000l), 				\
															(int)(TTicks/60000l) % 60,			\
															(int)(TTicks/1000l) % 60));


/*
* Container-Klasse TTopography fr Topographie-Parameter
* Konstruktor TTopography
* Dialog zum Ausfhren der Topographie: TTopographyExecute
* Dialog zum Setzen der Parameter: TTopographySetParam
*/

extern  LPDList		  lpDList;
extern	TSteering		Steering;

TTopography 	Topography;
TMacroTag* 		MWorkPoint;


//***** Parameter-Reprsentation fr die Topographie ***************************
TTopography::TTopography(void)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopography::TTopography(void)\n",time(zidx));
//pph***************************************************************************
	fMoveStep 						= 0.3;
	fMaxAngleEscape       = 50;     // arcsec
	lMeasurementTime      = 600;  	// Sekunden
	nNumberCycle      	  = 3;
	bMultibleShot         = FALSE;
	dStartAngle           = -20.0;  // arcsec
	fAngleBetweenShots    = 3.0;
	fFailure         	    = 0.02;
	bMonitorUsed      	  = FALSE;
	Sensor            	  = NULL;
	Monitor               = NULL;
	nMotor								= -1;
	};

void TTopography::Initialize(void)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopography::Initialize(void)\n",time(zidx));
//pph***************************************************************************
	char	Ident[] = "Topography";
	char	buf[MaxString];

	GetPrivateProfileString(Ident,"WorkPoint","-60.0",buf,MaxString,GetCFile());
	fWorkPoint            = fabs(atof(buf) / 100.0);
	bSmallAngleSide       = (atof(buf) < 0);
	GetPrivateProfileString(Ident,"ControlRange","3.0",buf,MaxString,GetCFile());
	fControlRange         = atof(buf) / 100.0;
	GetPrivateProfileString(Ident,"ControlStep","0.2",buf,MaxString,GetCFile());
	fControlStep					= atof(buf);
	GetPrivateProfileString(Ident,"ExposureTime","10.0",buf,MaxString,GetCFile());
	fMaxTime  						= atof(buf);
	GetPrivateProfileString(Ident,"ExposureCounts","10000",buf,MaxString,GetCFile());
	dwMaxCounts      	    = atol(buf);
	};

//*** Benutzer-Schnittstelle fr die Topographie *******************************
//******************************************************************************
TTopographyExecute::TTopographyExecute(void):TModalDlg("TopographyExecute")
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographyExecute::TTopographyExecute(void):TModalDlg(TopographyExecute)\n",time(zidx));
//pph***************************************************************************
	bStarted					= FALSE;
	bSetupOk 					= TRUE;
	bControlActive    = FALSE;
	bTimeRunning      = FALSE;
	bExeptionOccured  = FALSE;
	bAdditionalTime   = FALSE;
	if(Topography.Sensor == NULL)
		{
		Topography.Sensor  = lpDList->DP();
		Topography.Monitor = Device;
		Topography.bMonitorUsed = FALSE;
		bSetupOk = FALSE;
		}
	if(Topography.nMotor == -1)
		{
		Topography.nMotor  = mlGetIdByName(Omega);
		bSetupOk = FALSE;
		}
	};

#pragma argsused
BOOL TTopographyExecute::Dlg_OnInit(HWND hwnd,HWND hwndCtl,LPARAM lParam)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographyExecute::Dlg_OnInit(HWND hwnd,HWND hwndCtl,LPARAM lParam)\n",time(zidx));
//pph***************************************************************************
	char    buf[MaxString];

	// Detektor initialisieren
	Device = Topography.Sensor;
	nMotor = Topography.nMotor;
	if(!Device->bDeviceOpen)
		{
		NewWindow((TCounterWindow *)new TCounterWindow(Device->GetId()));
		}
	// initialize motor
	mlSetAxis(nMotor);
	nMotor = mlGetAxis();
	mPushSettings();
	CurrentTime = StartTime = 0;
	M_TimeString (buf,CurrentTime);
	SetDlgItemText(GetHandle(),id_MeasurementTime,(LPSTR)buf);
	sprintf(buf,"%.2f",0.0);
	SetDlgItemText(GetHandle(),id_Angle,(LPSTR)buf);
	sprintf(buf,"%d",Topography.nNumberCycle);
	SetDlgItemText(GetHandle(),id_RemainderCyclus,(LPSTR)buf);
	RestShots = Topography.nNumberCycle;
	if(Topography.bMultibleShot)
		SetDlgItemText(GetHandle(),cm_SwitchControl,"&Topographie starten");
		else
		SetDlgItemText(GetHandle(),cm_SwitchControl,"&Regeln starten");
	// Ablaufsteuerung initialisieren
	Steering.Visualising(1,0,0,0,0,0);
	MWorkPoint = Steering.GetMacroByName("SetupTopography");
	if(Topography.bSmallAngleSide)
		MWorkPoint->CmdList[1].P1 = SmallSide;
		else
		MWorkPoint->CmdList[1].P1 = LargeSide;
	sprintf(MWorkPoint->CmdList[1].P3,mGetDF(),fabs(Topography.fWorkPoint));
	SendMessage(GetHandle(),WM_COMMAND,cm_Initialize,0);
	SendMessage(GetHandle(),WM_COMMAND,cm_ParamSet,0);
	bStartPointOk = FALSE;
	bStarted = TRUE;
	return TRUE;
	};

void TTopographyExecute::Dlg_OnTimer(HWND hwnd,UINT id)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographyExecute::Dlg_OnTimer(HWND hwnd,UINT id)\n",time(zidx));
//pph***************************************************************************
	char    buf[MaxString];

	switch(id) {
		case 99:
		MessageBeep(0);
		DelayTime(100);
		MessageBeep(0);
		return;

		case 88:  // Timer fr Mehrfach-Belichtung
		FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
		if((CurrentTime/1000) < Topography.lMeasurementTime)
			return;
		KillTimer(GetHandle(),88);
		RestShots--;
		if(RestShots)
			{
			sprintf(buf,"%d",RestShots);
			SetDlgItemText(GetHandle(),id_RemainderCyclus,(LPSTR)buf);
			sprintf(buf,mGetDF(),Topography.fAngleBetweenShots);
			Steering.Reset();
			Steering.StartUp(GetHandle(),nMotor,Device->GetId());
			Steering.StartCmdExecution(MoveToPoint,Relative,0,buf,GetHandle());
			return;
			}
		sprintf(buf,"%d",RestShots);
		SetDlgItemText(GetHandle(),id_RemainderCyclus,(LPSTR)buf);
		bExeptionOccured = TRUE;
		SetTimer(GetHandle(),99,2000,NULL);
		bAdditionalTime = TRUE;
		strcpy(buf,"Me-Zeit abgelaufen ! \n\n");
		MessageBox(GetHandle(),buf,"Topographie",MBINFO);
		FORWARD_WM_COMMAND(hwnd,cm_SwitchControl,0,0,PostMessage);
		KillTimer(GetHandle(),99);
		break;
		}
	};

void TTopographyExecute::Dlg_OnCommand(HWND hwnd,int id,HWND hwndCtl,UINT codeNotify)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographyExecute::Dlg_OnCommand(HWND hwnd,int id,HWND hwndCtl,UINT codeNotify)\n",time(zidx));
//pph***************************************************************************
	TModalDlg*		dlg;
	static HWND   hDlgItem = NULL;
	char    			buf[2*MaxString];
	float					fExpTime;

	switch(id) {
		case cm_SwitchControl:
		// Starten der Regelung: Es wird zuerst die aktuelle Intensitt gemessen
		// und zum Regel-Sollwert erklrt. Dann erfolgt der Start des
		// Regelalgorithmus.
		//***** Topography was active **********************************************
		if(bControlActive)
			{
			if(Topography.bMultibleShot)
				KillTimer(GetHandle(),88);
				else
				Steering.ToggleInterrupt();
			// Herstellen des Ausgangszustandes
			bTimeFinish  = FALSE;
			bTimeRunning = FALSE;
			bAdditionalTime = FALSE;
			if(Topography.bMultibleShot)
				SetDlgItemText(GetHandle(),cm_SwitchControl,"&Topographie starten");
				else
				SetDlgItemText(GetHandle(),cm_SwitchControl,"&Regeln starten");
			StartTime = CurrentTime = 0;
			bControlActive = !bControlActive;
			SendMessage(GetHandle(),WM_COMMAND,cm_ParamSet,0);
			bControlActive = FALSE;
			EnableWindow(GetDlgItem(GetHandle(),cm_GotoWorkPoint),!bControlActive);
			break;
			}
		//***** Topography will be activated  **************************************
		bExeptionOccured  = FALSE;
		CurrentTime = 0;
		M_TimeString (buf,CurrentTime);
		SetDlgItemText(GetHandle(),id_RemainderTime,buf);
		StartTime = CurrentTime = GetTickCount();
		SetDlgItemText(GetHandle(),id_Text,"");
		bTimeRunning = TRUE;
		if(Topography.bMultibleShot)
			{
			// Fr Mehrfach-Belichtung
			// Start-Position wurde anfahren
			RestShots = Topography.nNumberCycle;
			SetTimer(GetHandle(),88,500,NULL);
			SetDlgItemText(GetHandle(),cm_SwitchControl,"&Topographie stoppen");
			}
			else
			{
			Steering.Reset();
			Steering.StartUp(GetHandle(),nMotor,Device->GetId());
			mSetValue(Width,Topography.fControlStep);
			StartTime = CurrentTime = GetTickCount();
			// Parameter fr Regelung laden
			sprintf(buf,mGetDF(),Topography.fControlRange);
			if(Topography.bSmallAngleSide)
				Steering.StartCmdExecution(ControlFlank,SmallSide,0,buf,GetHandle());
				else
				Steering.StartCmdExecution(ControlFlank,LargeSide,0,buf,GetHandle());
			SetDlgItemText(GetHandle(),cm_SwitchControl,"&Regeln stoppen");
			}
		bControlActive = TRUE;
		EnableWindow(GetDlgItem(GetHandle(),cm_GotoWorkPoint),!bControlActive);
		break;

		case cm_Initialize:
		if(Device->IsActive())
			{
			Device->MeasureStop();
			Device->SetExposureValues(	Topography.fMaxTime,Topography.dwMaxCounts, \
																Topography.fFailure );
			Device->MeasureStart();
			}
			else
			Device->SetExposureValues(	Topography.fMaxTime,Topography.dwMaxCounts, \
																Topography.fFailure );
		sprintf(buf,"%.1f",Topography.fMaxTime);
		SetDlgItemText(GetHandle(),id_ExposureTime,buf);
		RestShots = Topography.nNumberCycle;
		sprintf(buf,"%d",RestShots);
		SetDlgItemText(GetHandle(),id_RemainderCyclus,buf);
		mSetValue(Width,Topography.fControlStep);
		if(!bSetupOk)
			{
			strcpy(buf,"Die Einstellungen fr Topograpie knnten\n\r");
			strcat(buf,"noch nicht exakt eingestellt sein !\n\r");
			strcat(buf," Nehmen Sie bitte Ihre Einstellungen vor.");
			}
			else strcpy(buf,"");
		SetDlgItemText(GetHandle(),id_Text,(LPSTR)buf);
		break;

		case cm_GotoWorkPoint:
		// StartPosition anfahren
		strcpy(buf,"Arbeitspunkt wird angefahren.");
		SetDlgItemText(GetHandle(),id_Text,(LPSTR)buf);
		if(bStartPointOk && Topography.bMultibleShot)
			{
			// prevents the multible relative movement
			sprintf(buf,"Die Startposition wurde bereits angefahren !");
			MessageBox(GetHandle(),buf,"Topographie",MBINFO);
			break;
			}
		mSetValue(Width,Topography.fMoveStep);
		EnableWindow(GetDlgItem(GetHandle(),cm_SwitchControl),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),cm_GotoWorkPoint),FALSE);
		Steering.Reset();
		Steering.StartUp(GetHandle(),nMotor,Device->GetId());
		if(Topography.bMultibleShot)
			{
			sprintf(buf,mGetDF(),Topography.dStartAngle);
			Steering.StartCmdExecution(MoveToPoint,Relative,0,buf,GetHandle());
			}
			else
			Steering.StartMacroExecution(MWorkPoint,GetHandle());
		break;

		case cm_SetupPosition:
		// Arbeitspunkt wurde eingestellt
		EnableWindow(GetDlgItem(GetHandle(),cm_SwitchControl),TRUE);
		EnableWindow(GetDlgItem(GetHandle(),cm_GotoWorkPoint),TRUE);
		strcpy(buf,">>>>>>>>>>  X_RAY OFF  <<<<<<<<<<<<\n");
		strcat(buf,"Bitte Film-Kassette auflegen !\n");
		strcat(buf,"Die Aufnahme kann gestartet werden !");
		SetDlgItemText(GetHandle(),id_Text,(LPSTR)buf);
		bStartPointOk = TRUE;
		if(Topography.bMultibleShot)
			{
			Steering.dStartPoint = mGetValue(Distance) + Topography.dStartAngle;
			SendMessage(GetHandle(),WM_COMMAND,cm_ParamSet,0);
			}
		break;

		case cm_SteeringReady:
		if(bControlActive && Topography.bMultibleShot)
			{
			StartTime = CurrentTime = GetTickCount();
			bTimeRunning = TRUE;
			SetTimer(GetHandle(),88,500,NULL);
			break;
			}
		FORWARD_WM_COMMAND(GetHandle(),cm_SetupPosition,0,0,PostMessage);
		break;

		case cm_CounterSet:
		if(Topography.bMultibleShot)
			break;
		// Me-Werte speichern
		if(Topography.bMonitorUsed)
			{
			float norm;
			// Me-Signal normieren
			Topography.Monitor->GetData(norm);
			Steering.SetNorm(norm);
			}
		Device->UpdateDisplay();
		Steering.DeviceRequest();
		// Anzeige der Regel-Gren
		FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
		FORWARD_WM_COMMAND(GetHandle(),cm_InquireExeption,0,0,PostMessage);
		break;

		case id_ExposureTime:
		if(bStarted)
			hDlgItem = GetDlgItem(GetHandle(),id_ExposureTime);
		break;

		case IDOK:
		if(!hDlgItem)
			{
			TModalDlg::Dlg_OnCommand(hwnd,id,hwndCtl,codeNotify);
			break;
			}
		// Der Nutzer hat mit Return eine neue Mezeit eingegeben
		if(hDlgItem == GetDlgItem(GetHandle(),id_ExposureTime))
			{
			GetDlgItemText(GetHandle(),id_ExposureTime,(LPSTR)buf,10);
			fExpTime = atof(buf);
			Device->SetExposureValues( 	fExpTime,Topography.dwMaxCounts, \
																	Topography.fFailure );
			sprintf(buf,"%.1f",fExpTime);
			SetDlgItemText(GetHandle(),id_ExposureTime,(LPSTR)buf);
			}
		hDlgItem = NULL;
		SetFocus(GetDlgItem(GetHandle(),cm_SwitchControl));
		break;

		case cm_TopographyParam:
		// spezifische Parameter fr einen Sensor setzen
		dlg = (TTopographySetParam *)new TTopographySetParam(TRUE);
		dlg->ExecuteDialog(GetHandle());
		delete dlg;
		Device->MeasureStop();
		Device->SetExposureValues(Topography.fMaxTime,Topography.dwMaxCounts, \
																												Topography.fFailure);
		Device->MeasureStart();
		sprintf(buf,"%.1f",Topography.fMaxTime);
		SetDlgItemText(GetHandle(),id_ExposureTime,buf);
		mSetValue(Width,Topography.fControlStep);
		break;

		case cm_ParamSet:
		sprintf(buf,"%d",RestShots);
		SetDlgItemText(GetHandle(),id_RemainderCyclus,buf);
		if(!bControlActive)
			break;
		if(bTimeRunning)
			{
			CurrentTime = GetTickCount();
			CurrentTime -= StartTime;
			}
		sprintf(buf,"%.2lf",mGetValue(Distance) - Steering.dStartPoint);
		SetDlgItemText(GetHandle(),id_Angle,buf);
		if(bAdditionalTime)
			{
			// Ausgabe der zustzlichen Mezeit
			M_TimeString (buf,CurrentTime);
			SetDlgItemText(GetHandle(),id_RemainderTime,buf);
			}
			else
			{
			M_TimeString (buf,CurrentTime);
			SetDlgItemText(GetHandle(),id_MeasurementTime,buf);
			}
		break;

		case cm_InquireExeption:
		if(bExeptionOccured)
			break;
		// Feststellen, ob eine Abbruch-Bedingung erfllt ist
		if(Steering.GetIntensity() < (0.01 * Steering.fStartIntensity))
			{
			bExeptionOccured = TRUE;
			SetTimer(GetHandle(),99,2000,NULL);
			KillTimer(GetHandle(),TimerIdInformation);
			sprintf(buf,"%s","Intensitts-Abfall unter 1% !");
			MessageBox(GetHandle(),buf,"Topographie",MBINFO);
			KillTimer(GetHandle(),99);
			}
		if(Topography.bMultibleShot)
			break;
		if(Topography.fMaxAngleEscape <
				fabs(Steering.dStartPoint - Steering.GetDistance()))
			{
			bExeptionOccured = TRUE;
			KillTimer(GetHandle(),TimerIdInformation);
			sprintf(buf,"%s","Regel-Bereich wurde verlassen !");
			MessageBox(GetHandle(),buf,"Topographie",MBINFO);
			KillTimer(GetHandle(),99);
			}
		if((CurrentTime/1000) > Topography.lMeasurementTime)
			{
			bTimeFinish = TRUE;
			bExeptionOccured = TRUE;
			SetTimer(GetHandle(),99,2000,NULL);
			bAdditionalTime = TRUE;
			StartTime = GetTickCount();
			strcpy(buf,"Me-Zeit abgelaufen ! \n\n");
			strcat(buf,"Bei Besttigung wird Regelung gestoppt.");
			MessageBox(GetHandle(),buf,"Topographie",MBINFO);
			KillTimer(GetHandle(),99);
			FORWARD_WM_COMMAND(GetHandle(),cm_SwitchControl,0,0,PostMessage);
			}
		break;

		default:
		TModalDlg::Dlg_OnCommand(hwnd,id,hwndCtl,codeNotify);
		}
	};

BOOL TTopographyExecute::LeaveDialog(void)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographyExecute::LeaveDialog(void)\n",time(zidx));
//pph***************************************************************************
	KillTimer(GetHandle(),TimerIdInformation);
	KillTimer(GetHandle(),99);
	mPopSettings(ThisPosition);
	Device->MeasureStop();
	Device->PopSettings();
	Device->SetHost(NULL);
	Device->MeasureStart();
	if(!bSetupOk)
		Topography.Sensor = NULL;
	return TRUE;
	};

//******************************************************************************
TTopographySetParam::TTopographySetParam(int restrictions) : TModalDlg("TopographyParam")
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TModalDlg(TopographyParam)\n",time(zidx));
//pph***************************************************************************
	nRestrictions = restrictions;
	if(Topography.Sensor == NULL)
		{
		Topography.Sensor 			= lpDList->DP();
		Topography.Monitor 			= Topography.Sensor;
		Topography.bMonitorUsed = FALSE;
		}
	if(Topography.nMotor < 0)
		if(mlIsAxisValid(Omega))
			Topography.nMotor = mlGetIdByName(Omega);
			else
			Topography.nMotor = mlGetAxis();
	};

#pragma argsused
BOOL TTopographySetParam::Dlg_OnInit(HWND hwnd,HWND hwndCtl,LPARAM lParam)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographySetParam::Dlg_OnInit(HWND hwnd,HWND hwndCtl,LPARAM lParam)\n",time(zidx));
//pph***************************************************************************
	int		i;
	if(nRestrictions)
		{
		EnableWindow(GetDlgItem(GetHandle(),id_ChooseMotor),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),cm_RotateMotor),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_ChooseDevice),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_WorkPoint),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_Shots),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_MaxAngleEscape),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_StepWidth),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_From),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_MultibleShot),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_WorkPoint),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_ControlRange),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_MoveStep),FALSE);
		// EnableWindow(GetDlgItem(GetHandle(),id_ControlStep),FALSE);
		}
	nMotor  = Topography.nMotor;
	Device  = Topography.Sensor;
	Monitor = Topography.Monitor;
	hDeviceList = GetDlgItem(GetHandle(),id_ChooseDevice);
	hMotorList = GetDlgItem(GetHandle(),id_ChooseMotor);
	i = 0;
	while(i <= lpDList->LastId())
		ComboBox_AddString(hDeviceList,lpDList->DP(i++)->Characteristic);
	i = -1;
	while(++i < mlGetAxisNumber())
		{
		mlSetAxis(i);
		ComboBox_AddString(hMotorList,mGetAxisName());
		}
	mlSetAxis(nMotor);
	FORWARD_WM_COMMAND(hwnd,cm_ParamSet,hwndCtl,0,SendMessage);
  return TRUE;
	};

void TTopographySetParam::Dlg_OnCommand(HWND hwnd,int id,HWND hwndCtl,UINT codeNotify)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographySetParam::Dlg_OnCommand(HWND hwnd,int id,HWND hwndCtl,UINT codeNotify)\n",time(zidx));
//pph***************************************************************************
	char    buf[MaxString];
	float   valuef;
	int     idx;

	switch(id)
		{
		case cm_ParamSet:
		ComboBox_SelectString(hMotorList,0,mGetAxisName());
		ComboBox_SelectString(hDeviceList,0,Device->Characteristic);
		SetDlgItemText(GetHandle(),id_Unit,mGetAxisUnit());
		SetFocus(GetDlgItem(GetHandle(),IDCANCEL));
		valuef = Topography.fWorkPoint*100;
		if(Topography.bSmallAngleSide) valuef *= -1.0;
		sprintf(buf,"%.1f",valuef);
		SetDlgItemText(GetHandle(),id_WorkPoint,buf);
		SetDlgItemInt(GetHandle(),id_Shots,Topography.nNumberCycle,FALSE);
		sprintf(buf,"%.1f",Topography.fMaxAngleEscape);
		SetDlgItemText(GetHandle(),id_MaxAngleEscape,buf);
		// Schrittweite beim Regeln
		sprintf(buf,mGetSF(),Topography.fControlStep);
		SetDlgItemText(GetHandle(),id_ControlStep,buf);
		sprintf(buf,mGetSF(),Topography.fMoveStep);
		SetDlgItemText(GetHandle(),id_MoveStep,buf);
		sprintf(buf,"%.1f",Topography.fControlRange*100);
		SetDlgItemText(GetHandle(),id_ControlRange,buf);
		sprintf(buf,"%.1f",Topography.fMaxTime);
		SetDlgItemText(GetHandle(),id_MaxTime,buf);
		sprintf(buf,"%.d",(int)(Topography.lMeasurementTime/60l));
		SetDlgItemText(GetHandle(),id_MeasurementTime,buf);
		sprintf(buf,"%lu",Topography.dwMaxCounts);
		SetDlgItemText(GetHandle(),id_MaxCounts,buf);
		CheckDlgButton(GetHandle(),id_MultibleShot,Topography.bMultibleShot);
		sprintf(buf,"%.1f",Topography.fAngleBetweenShots);
		SetDlgItemText(GetHandle(),id_StepWidth,buf);
		sprintf(buf,"%.1f",Topography.dStartAngle);
		SetDlgItemText(GetHandle(),id_From,buf);
		if(!nRestrictions)
			{
			EnableWindow(GetDlgItem(GetHandle(),id_Shots),Topography.bMultibleShot);
			EnableWindow(GetDlgItem(GetHandle(),id_From),Topography.bMultibleShot);
			EnableWindow(GetDlgItem(GetHandle(),id_StepWidth),Topography.bMultibleShot);
			EnableWindow(GetDlgItem(GetHandle(),id_WorkPoint),!Topography.bMultibleShot);
			EnableWindow(GetDlgItem(GetHandle(),id_ControlRange),!Topography.bMultibleShot);
			}
		break;

		case id_ChooseMotor:
		switch(codeNotify)
			{
			case CBN_SELCHANGE:
			idx = ComboBox_GetCurSel(hMotorList);
			if((nMotor != idx) && (idx != CB_ERR))
				{
				mlSetAxis(idx);
				nMotor = mlGetAxis();
				FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
				}
			}
		break;

		case id_ChooseDevice:
		switch(codeNotify)
			{
			case CBN_SELCHANGE:
			idx = ComboBox_GetCurSel(hDeviceList);
			if((Device != lpDList->DP(idx)) && (idx != CB_ERR))
				{
				Device = lpDList->DP(idx);
				FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
				}
			}
		break;

		case cm_RotateMotor:
		mlSetAxis(nMotor+1);
		nMotor = mlGetAxis();
		FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
		break;

		case id_MultibleShot:
		Topography.bMultibleShot = IsDlgButtonChecked(GetHandle(),id_MultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_Shots),Topography.bMultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_From),Topography.bMultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_StepWidth),Topography.bMultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_WorkPoint),!Topography.bMultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_ControlRange),!Topography.bMultibleShot);
		break;

		default:
		TModalDlg::Dlg_OnCommand(hwnd,id,hwndCtl,codeNotify);
		}
	};

BOOL TTopographySetParam::CanClose(void)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographySetParam::CanClose(void)\n",time(zidx));
//pph***************************************************************************
	char    buf[MaxString];
	float   valueF;
	short   valueI;
	long    valueL;

	Topography.bMultibleShot = IsDlgButtonChecked(GetHandle(),id_MultibleShot);
	GetDlgItemText(GetHandle(),id_ControlRange,buf,10);
	valueF = atof(buf);
	if(0.0 != valueF)
		{
		Topography.fControlRange = valueF/100.0;
		}
		else return FALSE;
	GetDlgItemText(GetHandle(),id_StepWidth,buf,10);
	valueF = atof(buf);
	if(0.0 != valueF)
		{
		Topography.fAngleBetweenShots = valueF;
		}
		else return FALSE;

	GetDlgItemText(GetHandle(),id_From,buf,10);
	Topography.dStartAngle = atof(buf);

	GetDlgItemText(GetHandle(),id_ControlStep,buf,10);
	valueF = atof(buf);
	if(0.0 != valueF)
		{
		Topography.fControlStep = valueF;
		}
		else return FALSE;

	GetDlgItemText(GetHandle(),id_MoveStep,buf,10);
	valueF = atof(buf);
	if(0.0 != valueF)
		{
		Topography.fMoveStep = valueF;
		}
		else return FALSE;

	GetDlgItemText(GetHandle(),id_MaxAngleEscape,buf,10);
	valueF = atof(buf);
	if(0.0 != valueF)
		{
		Topography.fMaxAngleEscape = valueF;
		}
		else return FALSE;

	GetDlgItemText(GetHandle(),id_MeasurementTime,buf,10);
	valueL = atol(buf);
	if(0.0 != valueL)
		{
		Topography.lMeasurementTime = valueL * 60l;
		}
		else return FALSE;

	GetDlgItemText(GetHandle(),id_Shots,buf,10);
	valueI = (short)atoi(buf);
	if(0.0 != valueI)
		{
		Topography.nNumberCycle = valueI;
		}
		else return FALSE;

	GetDlgItemText(GetHandle(),id_WorkPoint,buf,10);
	valueF = atof(buf);
	if(0.0 != valueF)
		{
		Topography.bSmallAngleSide = (valueF < 0.0);
		Topography.fWorkPoint = fabs(valueF/100);
		}
		else return FALSE;
	GetDlgItemText(GetHandle(),id_MaxTime,buf,10);
	valueF = atof(buf);
	if(0.0 != valueF)
		{
		Topography.fMaxTime = valueF;
		}
		else return FALSE;
	GetDlgItemText(GetHandle(),id_MaxCounts,buf,10);
	valueL = atol(buf);
	if(0.0 != valueL)
		{
		Topography.dwMaxCounts = valueL;
		}
		else return FALSE;
	Topography.nMotor		= nMotor;
	Topography.Sensor   = Device;
	Topography.Monitor  = Monitor;
	return TRUE;
	};

BOOL TTopographySetParam::LeaveDialog(void)
	{
//hpp***************************************************************************
fprintf(trace,"%u		m_topo.cpp		TTopographySetParam::LeaveDialog(void)\n",time(zidx));
//pph***************************************************************************
	mlSetAxis(nOldMotor);
	return TRUE;
	};
//******************************************************************************
// __LastLine__
